"use strict";
/**
 * @fileoverview Prefer pascal case
 * @author Yann Braga
 */
const constants_1 = require("../utils/constants");
const create_storybook_rule_1 = require("../utils/create-storybook-rule");
module.exports = (0, create_storybook_rule_1.createStorybookRule)({
    name: 'prefer-pascal-case',
    defaultOptions: [],
    meta: {
        type: 'suggestion',
        fixable: 'code',
        docs: {
            description: 'Stories should use PascalCase',
            categories: [constants_1.CategoryId.RECOMMENDED],
            recommended: 'warn',
        },
        messages: {
            convertToPascalCase: 'Use pascal case',
            usePascalCase: 'The story should use PascalCase notation: {{name}}',
        },
        schema: [],
    },
    create(context) {
        // variables should be defined here
        //----------------------------------------------------------------------
        // Helpers
        //----------------------------------------------------------------------
        const isPascalCase = (str) => /^[A-Z]+([a-z0-9]?)+/.test(str);
        const toPascalCase = (str) => {
            return (str
                //@ts-ignore
                .replace(new RegExp(/[-_]+/, 'g'), ' ')
                //@ts-ignore
                .replace(new RegExp(/[^\w\s]/, 'g'), '')
                .replace(
            //@ts-ignore
            new RegExp(/\s+(.)(\w+)/, 'g'), ($1, $2, $3) => `${$2.toUpperCase() + $3.toLowerCase()}`)
                //@ts-ignore
                .replace(new RegExp(/\s/, 'g'), '')
                .replace(new RegExp(/\w/), (s) => s.toUpperCase()));
        };
        //----------------------------------------------------------------------
        // Public
        //----------------------------------------------------------------------
        return {
            ExportNamedDeclaration: function (node) {
                // if there are specifiers, node.declaration should be null
                if (!node.declaration)
                    return;
                const { type } = node.declaration;
                if (type === 'TSTypeAliasDeclaration' ||
                    type === 'TypeAlias' ||
                    type === 'TSInterfaceDeclaration' ||
                    type === 'InterfaceDeclaration') {
                    return;
                }
                const identifier = node.declaration.declarations[0].id;
                if (identifier) {
                    const { name } = identifier;
                    if (!isPascalCase(name)) {
                        context.report({
                            node: identifier,
                            messageId: 'usePascalCase',
                            data: { name },
                            suggest: [
                                {
                                    messageId: 'convertToPascalCase',
                                    fix: function (fixer) {
                                        return fixer.replaceTextRange(identifier.range, toPascalCase(name));
                                    },
                                },
                            ],
                        });
                    }
                }
            },
        };
    },
});
